searchData={"items":[{"type":"module","title":"Strom.Composite","doc":"Runs a set of components and is a component itself, meaning that a composite has the same interface - it accepts flow as input and returns a modified flow.","ref":"Strom.Composite.html"},{"type":"module","title":"Example - Strom.Composite","doc":"iex> alias Strom.{Composite, Transformer, Splitter, Source, Sink}\n    iex> transformer = Transformer.new(:s, &(&1 + 1))\n    iex> splitter = Splitter.new(:s, %{odd: &(rem(&1, 2) == 1), even: &(rem(&1, 2) == 0)})\n    iex> composite = [transformer, splitter] |> Composite.new() |> Composite.start()\n    iex> source = :s |> Source.new([1, 2, 3]) |> Source.start()\n    iex> %{odd: odd, even: even} = %{} |> Source.call(source) |> Composite.call(composite)\n    iex> {Enum.to_list(odd), Enum.to_list(even)}\n    {[3], [2, 4]}","ref":"Strom.Composite.html#module-example"},{"type":"module","title":"Composites can be created from other composites - Strom.Composite","doc":"iex> alias Strom.{Composite, Transformer, Splitter, Source, Sink}\n    iex> transformer = Transformer.new(:s, &(&1 + 1))\n    iex> splitter = Splitter.new(:s, %{odd: &(rem(&1, 2) == 1), even: &(rem(&1, 2) == 0)})\n    iex> c1 = Composite.new([transformer])\n    iex> c2 = Composite.new([splitter])\n    iex> source = Source.new(:s, [1, 2, 3])\n    iex> composite = [source, c1, c2] |> Composite.new() |> Composite.start()\n    iex> %{odd: odd, even: even} = %{} |> Composite.call(composite)\n    iex> {Enum.to_list(odd), Enum.to_list(even)}\n    {[3], [2, 4]}","ref":"Strom.Composite.html#module-composites-can-be-created-from-other-composites"},{"type":"function","title":"Strom.Composite.build/1","doc":"","ref":"Strom.Composite.html#build/1"},{"type":"function","title":"Strom.Composite.call/2","doc":"","ref":"Strom.Composite.html#call/2"},{"type":"function","title":"Strom.Composite.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Strom.Composite.html#child_spec/1"},{"type":"function","title":"Strom.Composite.new/1","doc":"","ref":"Strom.Composite.html#new/1"},{"type":"function","title":"Strom.Composite.reduce_flow/2","doc":"","ref":"Strom.Composite.html#reduce_flow/2"},{"type":"function","title":"Strom.Composite.start/1","doc":"","ref":"Strom.Composite.html#start/1"},{"type":"function","title":"Strom.Composite.stop/1","doc":"","ref":"Strom.Composite.html#stop/1"},{"type":"function","title":"Strom.Composite.stop_components/1","doc":"","ref":"Strom.Composite.html#stop_components/1"},{"type":"type","title":"Strom.Composite.t/0","doc":"","ref":"Strom.Composite.html#t:t/0"},{"type":"module","title":"Strom.DSL","doc":"DSL for building components","ref":"Strom.DSL.html"},{"type":"macro","title":"Strom.DSL.mix/3","doc":"","ref":"Strom.DSL.html#mix/3"},{"type":"macro","title":"Strom.DSL.rename/1","doc":"","ref":"Strom.DSL.html#rename/1"},{"type":"macro","title":"Strom.DSL.sink/3","doc":"","ref":"Strom.DSL.html#sink/3"},{"type":"macro","title":"Strom.DSL.source/2","doc":"","ref":"Strom.DSL.html#source/2"},{"type":"macro","title":"Strom.DSL.split/3","doc":"","ref":"Strom.DSL.html#split/3"},{"type":"macro","title":"Strom.DSL.transform/4","doc":"","ref":"Strom.DSL.html#transform/4"},{"type":"module","title":"Strom.GenMix","doc":"Generic functionality used by `Strom.Mixer` and `Strom.Splitter`.","ref":"Strom.GenMix.html"},{"type":"function","title":"Strom.GenMix.call/2","doc":"","ref":"Strom.GenMix.html#call/2"},{"type":"function","title":"Strom.GenMix.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Strom.GenMix.html#child_spec/1"},{"type":"function","title":"Strom.GenMix.start/1","doc":"","ref":"Strom.GenMix.html#start/1"},{"type":"function","title":"Strom.GenMix.start_link/1","doc":"","ref":"Strom.GenMix.html#start_link/1"},{"type":"function","title":"Strom.GenMix.stop/1","doc":"","ref":"Strom.GenMix.html#stop/1"},{"type":"module","title":"Strom.GenMix.Consumer","doc":"Consumer is used by the generic GenMix component.","ref":"Strom.GenMix.Consumer.html"},{"type":"function","title":"Strom.GenMix.Consumer.call/1","doc":"","ref":"Strom.GenMix.Consumer.html#call/1"},{"type":"function","title":"Strom.GenMix.Consumer.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Strom.GenMix.Consumer.html#child_spec/1"},{"type":"function","title":"Strom.GenMix.Consumer.start/3","doc":"","ref":"Strom.GenMix.Consumer.html#start/3"},{"type":"function","title":"Strom.GenMix.Consumer.stop/1","doc":"","ref":"Strom.GenMix.Consumer.html#stop/1"},{"type":"module","title":"Strom.Mixer","doc":"Mix several streams into one. Use Strom.GenMix under the hood","ref":"Strom.Mixer.html"},{"type":"module","title":"Example - Strom.Mixer","doc":"iex> alias Strom.Mixer\n    iex> mixer = [:s1, :s2] |> Mixer.new(:stream) |> Mixer.start()\n    iex> flow = %{s1: [1, 2, 3], s2: [4, 5, 6]}\n    iex> %{stream: stream} = Mixer.call(flow, mixer)\n    iex> stream |> Enum.to_list() |> Enum.sort()\n    [1, 2, 3, 4, 5, 6]","ref":"Strom.Mixer.html#module-example"},{"type":"module","title":"Can also accept a map with functions as values. Works like \"filter\". - Strom.Mixer","doc":"iex> alias Strom.Mixer\n    iex> inputs = %{s1: &(rem(&1, 2) == 0), s2: &(rem(&1, 2) == 1)}\n    iex> mixer = inputs |> Mixer.new(:stream) |> Mixer.start()\n    iex> flow = %{s1: [1, 2, 3], s2: [4, 5, 6]}\n    iex> %{stream: stream} = Mixer.call(flow, mixer)\n    iex> stream |> Enum.to_list() |> Enum.sort()\n    [2, 5]","ref":"Strom.Mixer.html#module-can-also-accept-a-map-with-functions-as-values-works-like-filter"},{"type":"function","title":"Strom.Mixer.call/2","doc":"","ref":"Strom.Mixer.html#call/2"},{"type":"function","title":"Strom.Mixer.new/2","doc":"","ref":"Strom.Mixer.html#new/2"},{"type":"function","title":"Strom.Mixer.start/2","doc":"","ref":"Strom.Mixer.html#start/2"},{"type":"function","title":"Strom.Mixer.stop/1","doc":"","ref":"Strom.Mixer.html#stop/1"},{"type":"type","title":"Strom.Mixer.event/0","doc":"","ref":"Strom.Mixer.html#t:event/0"},{"type":"type","title":"Strom.Mixer.t/0","doc":"","ref":"Strom.Mixer.html#t:t/0"},{"type":"module","title":"Strom.Renamer","doc":"Renames streams in flow.","ref":"Strom.Renamer.html"},{"type":"module","title":"Example - Strom.Renamer","doc":"iex> alias Strom.Renamer\n    iex> flow = %{s1: [1], s2: [2]}\n    iex> renamer = %{s1: :foo1, s2: :foo2} |> Renamer.new() |> Renamer.start()\n    iex> Renamer.call(flow, renamer)\n    %{foo1: [1], foo2: [2]}","ref":"Strom.Renamer.html#module-example"},{"type":"function","title":"Strom.Renamer.call/2","doc":"","ref":"Strom.Renamer.html#call/2"},{"type":"function","title":"Strom.Renamer.new/1","doc":"","ref":"Strom.Renamer.html#new/1"},{"type":"function","title":"Strom.Renamer.start/1","doc":"","ref":"Strom.Renamer.html#start/1"},{"type":"function","title":"Strom.Renamer.stop/1","doc":"","ref":"Strom.Renamer.html#stop/1"},{"type":"type","title":"Strom.Renamer.t/0","doc":"","ref":"Strom.Renamer.html#t:t/0"},{"type":"behaviour","title":"Strom.Sink","doc":"Runs a given steam and `call` origin on each even in stream.\nBy default it runs the stream asynchronously (in `Task.async`).\nOne can pass `true` a the third argument to the `Sink.new/3` function to run a stream synchronously.","ref":"Strom.Sink.html"},{"type":"behaviour","title":"Example - Strom.Sink","doc":"iex> alias Strom.{Sink, Sink.WriteLines}\n    iex> sink = :strings |> Sink.new(WriteLines.new(\"test/data/sink.txt\"), true) |> Sink.start()\n    iex> %{} = Sink.call(%{strings: [\"a\", \"b\", \"c\"]}, sink)\n    iex> File.read!(\"test/data/sink.txt\")\n    \"a\\nb\\nc\\n\"\n\nSink defines a `@behaviour`. One can easily implement their own sinks.\nSee `Strom.Sink.Writeline`, `Strom.Sink.IOPuts`, `Strom.Sink.Null`","ref":"Strom.Sink.html#module-example"},{"type":"function","title":"Strom.Sink.call/2","doc":"","ref":"Strom.Sink.html#call/2"},{"type":"callback","title":"Strom.Sink.call/2","doc":"","ref":"Strom.Sink.html#c:call/2"},{"type":"function","title":"Strom.Sink.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Strom.Sink.html#child_spec/1"},{"type":"function","title":"Strom.Sink.new/3","doc":"","ref":"Strom.Sink.html#new/3"},{"type":"function","title":"Strom.Sink.start/1","doc":"","ref":"Strom.Sink.html#start/1"},{"type":"callback","title":"Strom.Sink.start/1","doc":"","ref":"Strom.Sink.html#c:start/1"},{"type":"function","title":"Strom.Sink.start_link/1","doc":"","ref":"Strom.Sink.html#start_link/1"},{"type":"function","title":"Strom.Sink.stop/1","doc":"","ref":"Strom.Sink.html#stop/1"},{"type":"callback","title":"Strom.Sink.stop/1","doc":"","ref":"Strom.Sink.html#c:stop/1"},{"type":"type","title":"Strom.Sink.event/0","doc":"","ref":"Strom.Sink.html#t:event/0"},{"type":"type","title":"Strom.Sink.t/0","doc":"","ref":"Strom.Sink.html#t:t/0"},{"type":"module","title":"Strom.Sink.IOPuts","doc":"","ref":"Strom.Sink.IOPuts.html"},{"type":"function","title":"Strom.Sink.IOPuts.new/2","doc":"","ref":"Strom.Sink.IOPuts.html#new/2"},{"type":"module","title":"Strom.Sink.Null","doc":"","ref":"Strom.Sink.Null.html"},{"type":"function","title":"Strom.Sink.Null.new/0","doc":"","ref":"Strom.Sink.Null.html#new/0"},{"type":"module","title":"Strom.Sink.WriteLines","doc":"","ref":"Strom.Sink.WriteLines.html"},{"type":"function","title":"Strom.Sink.WriteLines.new/2","doc":"","ref":"Strom.Sink.WriteLines.html#new/2"},{"type":"behaviour","title":"Strom.Source","doc":"Produces stream of events.","ref":"Strom.Source.html"},{"type":"behaviour","title":"Example with Enumerable - Strom.Source","doc":"iex> alias Strom.Source\n    iex> source = :numbers |> Source.new([1, 2, 3]) |> Source.start()\n    iex> %{numbers: stream} = Source.call(%{}, source)\n    iex> Enum.to_list(stream)\n    [1, 2, 3]","ref":"Strom.Source.html#module-example-with-enumerable"},{"type":"behaviour","title":"Example with file - Strom.Source","doc":"iex> alias Strom.{Source, Source.ReadLines}\n    iex> source = :numbers |> Source.new(ReadLines.new(\"test/data/numbers1.txt\")) |> Source.start()\n    iex> %{numbers: stream} = Source.call(%{}, source)\n    iex> Enum.to_list(stream)\n    [\"1\", \"2\", \"3\", \"4\", \"5\"]","ref":"Strom.Source.html#module-example-with-file"},{"type":"behaviour","title":"If two sources are applied to one stream, the streams will be concatenated (Stream.concat/2) - Strom.Source","doc":"iex> alias Strom.{Source, Source.ReadLines}\n    iex> source1 = :numbers |> Source.new([1, 2, 3]) |> Source.start()\n    iex> source2 = :numbers |> Source.new(ReadLines.new(\"test/data/numbers1.txt\")) |> Source.start()\n    iex> %{numbers: stream} = %{} |> Source.call(source1) |> Source.call(source2)\n    iex> Enum.to_list(stream)\n    [1, 2, 3, \"1\", \"2\", \"3\", \"4\", \"5\"]\n\nSource defines a `@behaviour`. One can easily implement their own sources.\nSee `Strom.Source.ReadLines`, `Strom.Source.Events`, `Strom.Source.IOGets`","ref":"Strom.Source.html#module-if-two-sources-are-applied-to-one-stream-the-streams-will-be-concatenated-stream-concat-2"},{"type":"function","title":"Strom.Source.call/1","doc":"","ref":"Strom.Source.html#call/1"},{"type":"callback","title":"Strom.Source.call/1","doc":"","ref":"Strom.Source.html#c:call/1"},{"type":"function","title":"Strom.Source.call/2","doc":"","ref":"Strom.Source.html#call/2"},{"type":"function","title":"Strom.Source.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Strom.Source.html#child_spec/1"},{"type":"function","title":"Strom.Source.infinite?/1","doc":"","ref":"Strom.Source.html#infinite?/1"},{"type":"callback","title":"Strom.Source.infinite?/1","doc":"","ref":"Strom.Source.html#c:infinite?/1"},{"type":"function","title":"Strom.Source.new/2","doc":"","ref":"Strom.Source.html#new/2"},{"type":"function","title":"Strom.Source.start/1","doc":"","ref":"Strom.Source.html#start/1"},{"type":"callback","title":"Strom.Source.start/1","doc":"","ref":"Strom.Source.html#c:start/1"},{"type":"function","title":"Strom.Source.start_link/1","doc":"","ref":"Strom.Source.html#start_link/1"},{"type":"function","title":"Strom.Source.stop/1","doc":"","ref":"Strom.Source.html#stop/1"},{"type":"callback","title":"Strom.Source.stop/1","doc":"","ref":"Strom.Source.html#c:stop/1"},{"type":"type","title":"Strom.Source.event/0","doc":"","ref":"Strom.Source.html#t:event/0"},{"type":"type","title":"Strom.Source.t/0","doc":"","ref":"Strom.Source.html#t:t/0"},{"type":"module","title":"Strom.Source.Events","doc":"","ref":"Strom.Source.Events.html"},{"type":"function","title":"Strom.Source.Events.new/1","doc":"","ref":"Strom.Source.Events.html#new/1"},{"type":"module","title":"Strom.Source.IOGets","doc":"","ref":"Strom.Source.IOGets.html"},{"type":"function","title":"Strom.Source.IOGets.new/0","doc":"","ref":"Strom.Source.IOGets.html#new/0"},{"type":"module","title":"Strom.Source.ReadLines","doc":"","ref":"Strom.Source.ReadLines.html"},{"type":"function","title":"Strom.Source.ReadLines.new/1","doc":"","ref":"Strom.Source.ReadLines.html#new/1"},{"type":"module","title":"Strom.Splitter","doc":"Split a stream into several streams by applying given functions on events","ref":"Strom.Splitter.html"},{"type":"module","title":"Example - Strom.Splitter","doc":"iex> alias Strom.Splitter\n    iex> outputs = %{s1: &(rem(&1, 2) == 0), s2: &(rem(&1, 2) == 1)}\n    iex> splitter = :stream |> Splitter.new(outputs) |> Splitter.start()\n    iex> %{s1: s1, s2: s2} = Splitter.call(%{stream: [1, 2, 3]}, splitter)\n    iex> {Enum.to_list(s1), Enum.to_list(s2)}\n    {[2], [1, 3]}","ref":"Strom.Splitter.html#module-example"},{"type":"module","title":"Can also just duplicate a stream - Strom.Splitter","doc":"iex> alias Strom.Splitter\n    iex> splitter = :stream |> Splitter.new([:s1, :s2]) |> Splitter.start()\n    iex> %{s1: s1, s2: s2} = Splitter.call(%{stream: [1, 2, 3]}, splitter)\n    iex> {Enum.to_list(s1), Enum.to_list(s2)}\n    {[1, 2, 3], [1, 2, 3]}","ref":"Strom.Splitter.html#module-can-also-just-duplicate-a-stream"},{"type":"function","title":"Strom.Splitter.call/2","doc":"","ref":"Strom.Splitter.html#call/2"},{"type":"function","title":"Strom.Splitter.new/2","doc":"","ref":"Strom.Splitter.html#new/2"},{"type":"function","title":"Strom.Splitter.start/2","doc":"","ref":"Strom.Splitter.html#start/2"},{"type":"function","title":"Strom.Splitter.stop/1","doc":"","ref":"Strom.Splitter.html#stop/1"},{"type":"type","title":"Strom.Splitter.event/0","doc":"","ref":"Strom.Splitter.html#t:event/0"},{"type":"type","title":"Strom.Splitter.t/0","doc":"","ref":"Strom.Splitter.html#t:t/0"},{"type":"module","title":"Strom.Topology","doc":"Runs set of components. Restarts all of them in case of crash.\nProvides `start`, `stop`, `call` function on the module level.\nStarts topology as a process with the topology module name.\nSo only one topology with the given name can exist.","ref":"Strom.Topology.html"},{"type":"module","title":"Example: - Strom.Topology","doc":"iex>  defmodule OddEvenTopology do\n    ...>    use Strom.Topology\n    ...>    alias Strom.Sink.Null\n    ...>\n    ...>    def topology(_opts) do\n    ...>      [\n    ...>        Source.new(:s1, [1, 2, 3]),\n    ...>        Source.new(:s2, [4, 5, 6]),\n    ...>        Mixer.new([:s1, :s2], :s),\n    ...>        Transformer.new(:s, &(&1 + 1)),\n    ...>        Splitter.new(:s, %{odd: &(rem(&1, 2) == 1), even: &(rem(&1, 2) == 0)}),\n    ...>        Sink.new(:odd, Null.new(), true)\n    ...>      ]\n    ...>    end\n    ...>  end\n    iex> OddEvenTopology.start()\n    iex> %{even: even} = OddEvenTopology.call(%{})\n    iex> OddEvenTopology.stop()\n    iex> Enum.sort(Enum.to_list(even))\n    [2, 4, 6]","ref":"Strom.Topology.html#module-example"},{"type":"function","title":"Strom.Topology.call/2","doc":"","ref":"Strom.Topology.html#call/2"},{"type":"function","title":"Strom.Topology.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Strom.Topology.html#child_spec/1"},{"type":"function","title":"Strom.Topology.components/1","doc":"","ref":"Strom.Topology.html#components/1"},{"type":"function","title":"Strom.Topology.info/1","doc":"","ref":"Strom.Topology.html#info/1"},{"type":"function","title":"Strom.Topology.start/2","doc":"","ref":"Strom.Topology.html#start/2"},{"type":"function","title":"Strom.Topology.start_link/1","doc":"","ref":"Strom.Topology.html#start_link/1"},{"type":"function","title":"Strom.Topology.stop/1","doc":"","ref":"Strom.Topology.html#stop/1"},{"type":"module","title":"Strom.Transformer","doc":"Transforms a stream or several streams.\nIt works as Stream.map/2 or Stream.transform/3.\n\n    ## `map` example:\n    iex> alias Strom.Transformer\n    iex> transformer = :numbers |> Transformer.new(&(&1*2)) |> Transformer.start()\n    iex> flow = %{numbers: [1, 2, 3]}\n    iex> %{numbers: stream} = Transformer.call(flow, transformer)\n    iex> Enum.to_list(stream)\n    [2, 4, 6]\n\n    ## `transform` example:\n    iex> alias Strom.Transformer\n    iex> fun = fn el, acc -> {[el, acc], acc + 10} end\n    iex> transformer = :numbers |> Transformer.new(fun, 10) |> Transformer.start()\n    iex> flow = %{numbers: [1, 2, 3]}\n    iex> %{numbers: stream} = Transformer.call(flow, transformer)\n    iex> Enum.to_list(stream)\n    [1, 10, 2, 20, 3, 30]","ref":"Strom.Transformer.html"},{"type":"module","title":"it can be applied to several streams: - Strom.Transformer","doc":"iex> alias Strom.Transformer\n    iex> transformer = [:s1, :s2] |> Transformer.new(&(&1*2)) |> Transformer.start()\n    iex> flow = %{s1: [1, 2, 3], s2: [4, 5, 6]}\n    iex> %{s1: s1, s2: s2} = Transformer.call(flow, transformer)\n    iex> {Enum.to_list(s1), Enum.to_list(s2)}\n    {[2, 4, 6], [8, 10, 12]}","ref":"Strom.Transformer.html#module-it-can-be-applied-to-several-streams"},{"type":"function","title":"Strom.Transformer.call/2","doc":"","ref":"Strom.Transformer.html#call/2"},{"type":"function","title":"Strom.Transformer.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Strom.Transformer.html#child_spec/1"},{"type":"function","title":"Strom.Transformer.new/3","doc":"","ref":"Strom.Transformer.html#new/3"},{"type":"function","title":"Strom.Transformer.start/2","doc":"","ref":"Strom.Transformer.html#start/2"},{"type":"function","title":"Strom.Transformer.start_link/1","doc":"","ref":"Strom.Transformer.html#start_link/1"},{"type":"function","title":"Strom.Transformer.stop/1","doc":"","ref":"Strom.Transformer.html#stop/1"},{"type":"type","title":"Strom.Transformer.acc/0","doc":"","ref":"Strom.Transformer.html#t:acc/0"},{"type":"type","title":"Strom.Transformer.event/0","doc":"","ref":"Strom.Transformer.html#t:event/0"},{"type":"type","title":"Strom.Transformer.func/0","doc":"","ref":"Strom.Transformer.html#t:func/0"},{"type":"type","title":"Strom.Transformer.t/0","doc":"","ref":"Strom.Transformer.html#t:t/0"}],"content_type":"text/markdown"}